/*========================================================================*
 * TITLE:	Qwik Mail Reader for Unix
 * MODULE:	unpack.c
 * BY:		Ross C LINDER	(c) 1994 All rights reserved
 *
 * X11-QMR is Copyright (c) 1994 of Ross C Linder. X11-QMR is not public
 * domain. Permission is granted to use and distribute X11-QMR freely.
 * 
 * The only restriction is that you may not distibute modified source
 * code, and that you do not attempt to prevent others from having free
 * access to the source.
 *========================================================================*/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <Xm/SelectioB.h>
#include <Xm/List.h>
#include "util.h"
#include "qwk.h"

static char id[]=

"@(#)unpack.c	$Revision: 1.1 $	$Date: 1994/07/24 17:49:35 $";

/*========================================================================*/

Widget		conf_sel = NULL, conf_list;
Widget		msg_sel = NULL, msg_list;

extern Widget	fmsg_pb ;
extern Widget	bmsg_pb ;
extern Widget	fconf_pb;
extern Widget	bconf_pb;
extern Widget	toplevel, econtrol_box;

char		*xmsstr ();

int	conf_skip_flg = 0;	/* 1 = Dont Skip Empty confs */

char	bbs_tagfile[256] = "~/QMR/taglines";
char	bbs_printcmd[256] = "lp";
char	bbs_spellcmd[256] = "xterm -e ispell";
char	bbs_xeditcmd[256] = "xterm -e vi";
char	bbs_path[256];
char	bbs_user[26];
char	bbs_name[26];
char	bbs_news[10][20];	/* News flash files */
QWKHDR	extr_hdr;

struct {
	char	name[30];
	short	num;
	int	nmsg;
	int	npermsg;
	} qwk_areas[9999];	/* Max is 8191 */


static		max_conf = 0;
static		cur_conf = 0;
static short	cur_msg = 0;

/*========================================================================*
 * MAKE_BBS_PATH -- set the bbs_path string to ~/QMR/xxxx
 *========================================================================*/

make_bbs_path (char *fname)
{
int	a;

    a = strlen (fname);
    while (fname[a] != '/' && a)
	a--;

    if (a)
	a++;

    strcpy (bbs_name, &fname[a]);

    a = strlen (bbs_name);
    while (bbs_name[a] != '.' && a)
	a--;

    if (a)
	bbs_name[a] = 0;

    sprintf (bbs_path, "%s/QMR/%s", getenv ("HOME"), bbs_name);

    cur_conf = -1;
    cur_msg = 0;
}

/*========================================================================*
 * UNPACK_QWK -- Unpack the terrible QWK mail packet
 *
 * NOTE: This uses unzip !
 *========================================================================*/

unpack_qwk (char *fname, int do_data) 
{
FILE	*fpi;
char	ebuf[256];
int	a;
Widget	work_wid;

    if ((fpi = fopen (fname, "r")) == NULL) {
	sprintf (ebuf, "Error: %s\n       %s", fname, strerror (errno));
	error_dialog (ebuf, "OK", "", "", NULL);
	return -1;
	}

    fclose (fpi);

    mkdir (bbs_path, 0776);
    chdir (bbs_path);

    work_wid = working_dialog ("Uncompressing qwk packet", "", "", "", NULL);

    if (do_data) {
	system ("rm -f *.cnf *.qmr *.dat");	/* Incase of a fuckup */

	sprintf (ebuf, "unzip -qq -o %s", fname);
	system (ebuf);
	XtUnmanageChild (work_wid);
	}

    chdir (getenv ("HOME"));

    if (read_control () == -1)
	return -1;

    process_data (do_data);

    if (conf_sel != NULL) {
	if (XtIsManaged (conf_sel))
	    view_conf_sel ();
	}

    if (msg_sel != NULL) {
	if (XtIsManaged (msg_sel))
	    view_msg_sel ();
	}
    return 0;
}

/*========================================================================*
 * READ_CONTROL -- Read the qwk control file 
 *========================================================================*/

read_control ()
{
char	fname[256];
FILE	*fp;
char	tbuf[80];
int	a, b;
int	area_count;
int	num_areas;

    strcpy (fname, bbs_path);
    strcat (fname, "/control.dat");

    if ((fp = fopen (fname, "r")) == NULL) {
	sprintf (tbuf, "Error: %s\n       %s", fname, strerror (errno));
	error_dialog (tbuf, "OK", "", "", NULL);
	return -1;
	}

    for (a = 0; a < 5; a++)
	fgets (tbuf, 80, fp);

    b = strlen (tbuf) -  2;			/* Wipe out CRLF */
    tbuf[b] = 0;

    while (--b)
	if (tbuf[b] == ',') {
	    b++;
	    break;
	    }

    strcpy (bbs_name, &tbuf[b]);

    for (; a < 7; a++)
	fgets (tbuf, 80, fp);

    strcpy (bbs_user, tbuf);
    bbs_user[strlen (bbs_user) - 2] = 0;	/* Wipe out CRLF */

    for (; a < 10; a++)
	fgets (tbuf, 80, fp);

    fgets (tbuf, 80, fp);
    num_areas = atoi(tbuf) + 1;

    area_count = 0;
    for (area_count = 0; area_count < num_areas; area_count++) {
	if (!fgets (tbuf, 80, fp))
	    break;
	qwk_areas[area_count].num = atoi(tbuf);

	if (!fgets (tbuf, 80, fp))
	    break;
	a = strlen (tbuf) - 2;
	tbuf[a] = 0;		/* Kill CR LF */
	while (a) {
	    if (isspace (tbuf[a]))
		tbuf[a] = 0;
	    else
		break;
	    a--;
	    }
	tbuf[30] = 0;
	strcpy (qwk_areas[area_count].name, tbuf);
	}
	
    if (area_count != num_areas) {
	error_dialog ("Error: Control file is corrupt", "", "", "", NULL);
	return -1;
	}

    b = 0;
    while (fgets (tbuf, 80, fp)) {
	if (tbuf[0] == '0' || tbuf[0] == 26 /* ^Z */)
	    break;

	for (a = 0; a < strlen (tbuf) - 2; a++)
	    tbuf[a] = tolower (tbuf[a]);
	tbuf[a] = 0;

	if (a > 20)		/* Must be dos compat filename */
	    continue;
		
	strcpy (bbs_news[b++], tbuf);
	}

    while (b < 10)
	bbs_news[b++][0] = 0;

    fclose (fp);

    strcpy (fname, bbs_path);
    strcat (fname, "/index.qmr");

    if ((fp = fopen (fname, "w")) == NULL) {
	sprintf (tbuf, "Error: %s\n       %s", fname, strerror (errno));
	error_dialog (tbuf, "OK", "", "", NULL);
	return -1;
	}

    fwrite ("\0", 1, 4 * num_areas, fp);	/* Sim Short of 0 */

    fclose (fp);

    max_conf = num_areas;

    return 0;
}

/*========================================================================*
 * PROCESS_DATA -- Make conference data files
 *
 * I have received some really corrupt mail packets from Qmail door, so
 * I have tried to do "error recovery" if I get junk :-)
 *========================================================================*/

process_data (int do_data)
{
char		fname[256];
FILE		*fp, *fpc[9999];
int		cnf_msg_cnt[9999];
int		per_msg_cnt[9999];
char		tbuf[QWK_BLK+1];	/* To null terminate */
int		a, b;
long		area_count;
QWKHDR		hdr;
int		last_area = -1;
Widget		work_wid;
long		hpos;
int		errcnt = 0;
int		errf = 0;

    for (a = 0; a < 9999; a++) {
	cnf_msg_cnt[a] = 0;
	per_msg_cnt[a] = 0;
	fpc[a] = NULL;
	qwk_areas[a].nmsg = 0;
	qwk_areas[a].npermsg = 0;
	}

    strcpy (fname, bbs_path);
    strcat (fname, "/messages.dat");

    if ((fp = fopen (fname, "r+")) == NULL) {
	sprintf (tbuf, "Error: %s\n       %s", fname, strerror (errno));
	error_dialog (tbuf, "OK", "", "", NULL);
	return;
	}

    read_qwk_hdr (fp, &hdr, 0, 0);	/* waste Sparks (c) */

    hpos = 1;		/* Just in case we are fed shit */

    for (;;) {
	if (!read_qwk_hdr (fp, &hdr, 0, errf))
	    break;

	if ((hdr.status != ' ' && hdr.status != '-' && hdr.status != '+' &&
	    hdr.status != '*' && hdr.status != '~' &&
	    hdr.status != '`' && hdr.status != '%' &&
	    hdr.status != '^' && hdr.status != '!' &&
	    hdr.status != '#' && hdr.status != '$') ||

	    hdr.numchunks < 1 || hdr.msgnum < 0 || hdr.confnum > 9000) {
		errcnt++;
		if (errcnt == 2) {
		     warn_dialog ("Warning: QWK packet corrupt, trying "
		     "error recovery ...", "OK", "", "", NULL);
		     }

	    fseek (fp, hpos, SEEK_SET);
	    hpos++;
	    if (hpos == 0xA5F1)
		printf ("Ahha\n");

	    errf = 1;
	    continue;
	    }

	errf = 0;
	hpos = ftell (fp);		/* For Error recovery */

	if (last_area != hdr.confnum) {
	    sprintf (tbuf, "Processing Conference %d", hdr.confnum);
	    work_wid = working_dialog (tbuf, "", "", "", NULL);
	    last_area = hdr.confnum;
	    }

	cnf_msg_cnt[hdr.confnum]++;
	if (strcmp (hdr.to, bbs_user) == 0)
	    per_msg_cnt[hdr.confnum]++;

	if (fpc[hdr.confnum] == NULL && do_data) {
	    sprintf (fname, "%s/%d.cnf", bbs_path, hdr.confnum);
	    fpc[hdr.confnum] = fopen (fname, "w");
	    }

	if (fpc[hdr.confnum] != NULL)
	    fwrite (&hdr, sizeof (hdr), 1, fpc[hdr.confnum]);

	for (a = 0; a < hdr.numchunks; a++) {
	    read_qwk_blk (fp, tbuf);

	    if (do_data)
		fwrite (tbuf, QWK_BLK, 1, fpc[hdr.confnum]);
	    }
	}

    if (errcnt > 1) {
	sprintf (tbuf, "Warning, the QWK packet was corrupt,"
	" some data was lost !\n"
	"It took %d error recovery tries to get back in sync", errcnt);
	
	warn_dialog (tbuf, "OK", "", "", NULL);
	}

    fclose (fp);

    for (a = 0; a < 9999; a++)
	if (fpc[a] != NULL) {
	    fclose (fpc[a]);
	    }

    XtUnmanageChild (work_wid);

    fp = fopen ("/tmp/stats", "w");

    fprintf (fp, " Conf   Description                  Messages  Personal\n");


    for (a = 0; a < 9999; a++) {
	if (!cnf_msg_cnt[a])
	    continue;

	for (b = 0; b < 9999; b++)
	    if (qwk_areas[b].num == a) {
		qwk_areas[b].nmsg = cnf_msg_cnt[a];
		qwk_areas[b].npermsg = per_msg_cnt[a];
		break;
		}

	fprintf (fp, "%4d     %-30.30s %4d      %4d\n", qwk_areas[b].num,
		qwk_areas[b].name, qwk_areas[b].nmsg, qwk_areas[b].npermsg);
	}

    fclose (fp);

    set_header_display (bbs_user, "X11-QMR", "Message Stats",
			time (0), "", 1, 1);

    file_to_msg ("/tmp/stats", 1);

    if (do_data) {
	strcpy (fname, bbs_path);
	strcat (fname, "/");
	strcat (fname, bbs_name);
	strcat (fname, ".msg");

	if ((fp = fopen (fname, "w")) == NULL) {
	    sprintf (tbuf, "Error: %s\n       %s", fname, strerror (errno));
	    error_dialog (tbuf, "OK", "", "", NULL);
	    return;
	    }
	memset (tbuf, ' ', QWK_BLK);
	memcpy (tbuf, bbs_name, strlen (bbs_name));
	fwrite (tbuf, QWK_BLK, 1, fp);
	fclose (fp);
	}
}


/*========================================================================*
 * EXTRACT_MSG -- Extract a message from the conf file
 *========================================================================*/

extract_msg (int msg)
{
int	a, rdf;
char	fname[256], wname[256];
FILE	*fpi, *fpo;
char	ebuf[256], tbuf[256];
long	hpos;
int	spos, sposn;

    strcpy (wname, bbs_path);
    strcat (wname, "/work");

    sprintf (fname, "%s/%d.cnf", bbs_path, qwk_areas[cur_conf].num);

    if ((fpo = fopen (wname, "w")) == NULL) {
	sprintf (ebuf, "Error: %s\n       %s", fname, strerror (errno));
	error_dialog (ebuf, "OK", "", "", NULL);
	return - 1;
	}

    if ((fpi = fopen (fname, "r+")) == NULL) {
	fprintf (fpo, "No messages in conference\n");
	fclose (fpo);
	}
    else {
	for (a = 0; a < msg; a++) {
	    fread (&extr_hdr, sizeof (extr_hdr), 1, fpi);
	    while (extr_hdr.numchunks) {
		fread (tbuf, QWK_BLK, 1, fpi);
		extr_hdr.numchunks--;
		}
	    }

	hpos = ftell (fpi);

	if (fread (&extr_hdr, sizeof (extr_hdr), 1, fpi) <= 0) {
	    fclose (fpi);
	    fclose (fpo);
	    return -1;
	    }

	rdf = extr_hdr.live == '-';


	if (!rdf) {
	    fseek (fpi, hpos, SEEK_SET);
	    extr_hdr.live = '-';
	    fwrite (&extr_hdr, sizeof (extr_hdr), 1, fpi);
	    fseek (fpi, hpos + sizeof (extr_hdr), SEEK_SET);
	    }


	for (a = 0; a < extr_hdr.numchunks; a++) {
	    fread (tbuf, QWK_BLK, 1, fpi);
	    fwrite (tbuf, QWK_BLK, 1, fpo);
	    }

	fclose (fpo);
	fclose (fpi);
	}

    file_to_msg (wname, 1);

    if (qwk_areas[cur_conf].nmsg == 0) {
	strcpy (extr_hdr.to, bbs_user);
	strcpy (extr_hdr.from, "X11-QMR");
	strcpy (extr_hdr.subj, "Mail packet empty");
	extr_hdr.date = time (0);
	extr_hdr.confnum = qwk_areas[cur_conf].num;
	extr_hdr.msgnum = 0;
	}

    display_header (&extr_hdr);

    if (msg_sel != NULL) {
	if (XtIsManaged (msg_sel)) {
	    if (qwk_areas[cur_conf].nmsg) {
		XmListSelectPos (msg_list, msg+1, 0);	/* Dont notify */
		XmListSetBottomPos (msg_list, msg+1);
		}
	    }
	}

    return qwk_areas[cur_conf].nmsg ? 0 : -1;
}


/*========================================================================*
 * DISPLAY_HEADER -- Display a message header with conf name and Read flag
 *========================================================================*/

display_header (QWKHDR *hdr)
{
int	a, rdf;
char	tbuf[256];

    rdf = hdr->live == '-';

    for (a = 0; a < 9999; a++)
	if (hdr->confnum == qwk_areas[a].num)
	    break;

    if (a == 9999) {
	error_dialog ("QWK Packet is corrupt", "OK", "", "", NULL);
	printf ("hdr conf is %d\n", hdr->confnum);
	a = 0;
	}

    sprintf (tbuf, "%d  %s", hdr->confnum, qwk_areas[a].name);

    set_header_display (hdr->to, hdr->from, hdr->subj,
			hdr->date, tbuf, hdr->msgnum, rdf);
}

/*========================================================================*
 * NEXT_MESSAGE -- Advance to the next message if posible, else beep
 *========================================================================*/

next_message ()
{
FILE	*fpid;
char	fname[256], ebuf[256];
short	cpos;

    if (extract_msg (cur_msg+1) != -1) {
	cur_msg++;
	strcpy (fname, bbs_path);
	strcat (fname, "/index.qmr");
	if ((fpid = fopen (fname, "r+")) == NULL) {
	    sprintf (ebuf, "Error: %s\n       %s", fname, strerror (errno));
	    error_dialog (ebuf, "OK", "", "", NULL);
	    return 0;
	    }

	fseek (fpid, qwk_areas[cur_conf].num * 4, SEEK_SET);
	fread (&cpos, 4, 1, fpid);
	if (cpos < cur_msg) {
	    fseek (fpid, qwk_areas[cur_conf].num * 4, SEEK_SET);
	    fwrite (&cur_msg, 4, 1, fpid);
	    }
	fclose (fpid);
	}
    else {
	fputc (7, stderr);
	return 0;
	}

    return 1;
}

/*========================================================================*
 * PREV_MESSAGE -- Backtrack to the prev message if posible, else beep
 *========================================================================*/

prev_message ()
{
    if (cur_msg == 0) {		/* At first msg */
	fputc (7, stderr);
	return 0;
	}

    if (extract_msg (cur_msg-1) != -1)
	cur_msg--;

    return 1;
}

/*========================================================================*
 * NEXT_CONF -- Advance to the next conference. Skip empty confs,
 *              unless user requested otherwise. Beep if at END.
 *========================================================================*/

next_conf ()
{
int	a, b;

    if (cur_conf + 1 == max_conf) {
	fputc (7, stderr);
	return 0;
	}

    if (conf_skip_flg == 0) {
	b = cur_conf;
	while (cur_conf++ <= max_conf) {
	    if (qwk_areas[cur_conf].nmsg)
		break;
	    }
	if (cur_conf > max_conf) {
	    cur_conf = b;
	    return 0;
	    }
	}
    else
	cur_conf++;

    cur_msg = 0;

    if (msg_sel != NULL) {
	if (XtIsManaged (msg_sel))
	    view_msg_sel ();
	}

    extract_msg (0);

    return 1;
}

/*========================================================================*
 * PREV_CONF -- Backtrack to the prev conference. Skip empty confs,
 *              unless user requested otherwise. Beep if at START.
 *========================================================================*/

prev_conf ()
{
int	a;

    if (cur_conf == 0) {
	fputc (7, stderr);
	return 0;
	}

    if (conf_skip_flg == 0) {
	a = cur_conf;
	while (cur_conf-- >= 0)
	    if (qwk_areas[cur_conf].nmsg)
		break;

	if (cur_conf < 0) {
	    cur_conf = a;
	    return 0;
	    }
	}
    else 
	cur_conf--;

    cur_msg = 0;

    if (msg_sel != NULL) {
	if (XtIsManaged (msg_sel))
	    view_msg_sel ();
	}

    extract_msg (0);
    return 1;
}

/*========================================================================*
 * PACK_REPLIES -- Zip the reply message packet.
 *
 * NOTE: This requires zip !
 *========================================================================*/

pack_replies ()
{
char		fname[256];
char		zname[256];
char		cmd[256];
struct stat	st;
int		a;
Widget		w;

    strcpy (fname, bbs_path);
    strcat (fname, "/");
    strcat (fname, bbs_name);
    strcat (fname, ".msg");

    if (stat (fname, &st) == -1)
	return;

    if (st.st_size <= QWK_BLK)
	return;

    for (a = 0; bbs_name[a]; a++)
	cmd[a] = tolower (bbs_name[a]);
    cmd[a] = 0;

    sprintf (zname, "%s/QMR/UP/%s.rep", getenv ("HOME"), cmd);

    w = working_dialog ("Compressing Replies", "", "", "", NULL);

    sprintf (cmd, "zip -q -j %s %s", zname, fname);
    system (cmd);

    XtUnmanageChild (w);
}

/*========================================================================*
 * EBUSY_ERROR -- Error Dialog when busy editing
 *========================================================================*/

ebusy_error ()
{
    if (XtIsManaged (econtrol_box)) {
	error_dialog ("Finish your edit first !" , "OK", "", "", NULL);
	return 1;
	}

    return 0;
}

/*========================================================================*
 * CONF_PRESS -- Selection from conference selection list
 *
 * Here we check txt due to possibly empty conf's
 *========================================================================*/

void conf_press (Widget w, int ud, XmSelectionBoxCallbackStruct *cbd)
{
int	a, cnum;
char	*s, ebuf[256];

    if (!cbd->value)
	return;

   if (ebusy_error ())
	return;

    s = xmsstr (cbd->value);

    cnum = atoi (s);

    for (a = 0; a <= max_conf; a++) {
	if (qwk_areas[a].num == cnum)
	    break;
	}

    if (a == max_conf + 1) {
	error_dialog ("Invalid conference", "OK", "", "", NULL);
	a = cur_conf;
	}

    XtFree (s);

    if (a != cur_conf) {
	cur_conf = a;
	cur_msg = 0;
	if (msg_sel != NULL) {
	    if (XtIsManaged (msg_sel))
		view_msg_sel ();
	    }
	extract_msg (0);
	}
}

/*========================================================================*
 * MSG_PRESS -- Selection from message selection list
 *========================================================================*/

void msg_press (Widget w, int ud, XmSelectionBoxCallbackStruct *cbd)
{
int	a;
char	ebuf[256];

    if (!cbd->value)
	return;

   if (ebusy_error ())
	return;

    if (!XmListItemExists (msg_list, cbd->value)) {	/* A smart, arse */
	error_dialog ("Invalid message", "OK", "", "", NULL);
	a = cur_msg;
	}
    else
	a = XmListItemPos (msg_list, cbd->value);

    if (a != cur_msg) {
	cur_msg = a;
	extract_msg (a);
	}
}


/*========================================================================*
 * CONF_SEL_SEL -- Selection from message selection list with OK
 *========================================================================*/

void conf_sel_sel (Widget w, int ud, XmListCallbackStruct *cbd)
{
Arg	args[2];
int	a, b;

   if (ebusy_error ())
	return;

   if (fconf_pb != NULL) {
	XtSetArg (args[0], XmNsensitive, 1);
	XtSetValues (fconf_pb, args, 1);
	fconf_pb = NULL;
	}
   if (bconf_pb != NULL) {
	XtSetArg (args[0], XmNsensitive, 1);
	XtSetValues (bconf_pb, args, 1);
	bconf_pb = NULL;
	}

    a = cur_conf;

    if (conf_skip_flg == 0) {
	b = 0;
	cur_conf = -1;
	while (b < cbd->item_position) {
	    cur_conf++;
	    if (qwk_areas[cur_conf].nmsg)
		b++;
	    }
	}
    else
	cur_conf = cbd->item_position - 1;

    if (msg_sel != NULL) {
	if (XtIsManaged (msg_sel))
	    view_msg_sel ();
	}

    cur_msg = 0;
    extract_msg (0);
}

/*========================================================================*
 * MSG_SEL_SEL -- Selection from conference selection list with OK
 *========================================================================*/

void msg_sel_sel (Widget w, int ud, XmListCallbackStruct *cbd)
{
Arg	args[2];

   if (ebusy_error ())
	return;

   if (fmsg_pb != NULL) {
	XtSetArg (args[0], XmNsensitive, 1);
	XtSetValues (fmsg_pb, args, 1);
	fconf_pb = NULL;
	}
   if (bmsg_pb != NULL) {
	XtSetArg (args[0], XmNsensitive, 1);
	XtSetValues (bmsg_pb, args, 1);
	bconf_pb = NULL;
	}

    cur_msg = cbd->item_position - 1;
    extract_msg (cur_msg);
}

/*========================================================================*
 * VIEW_CONF_SEL -- Create/Manage the view conferences dialog
 *========================================================================*/

view_conf_sel ()
{
Arg		args[5];
int		a;
char		tbuf[256];
Widget		twid;
XmString	xms;

    if (conf_sel == NULL) {	/* Not yet created */
	xms = XmStringCreateLtoR ("Conference Selection",
				XmFONTLIST_DEFAULT_TAG);
	XtSetArg (args[0], XmNdialogTitle, xms);
	XtSetArg (args[1], XmNresizePolicy, XmRESIZE_NONE);
	conf_sel = XmCreateSelectionDialog (toplevel, "Sel", args, 2);
	XmStringFree (xms);

	conf_list = XmSelectionBoxGetChild (conf_sel, XmDIALOG_LIST);
	XtAddCallback (conf_sel, XmNokCallback,
			(XtCallbackProc)conf_press, 0);
	XtAddCallback (conf_sel, XmNapplyCallback,
			(XtCallbackProc)conf_press, 0);
	XtAddCallback (conf_list, XmNsingleSelectionCallback,
			(XtCallbackProc)conf_sel_sel, 0);
	XtAddCallback (conf_list, XmNbrowseSelectionCallback,
			(XtCallbackProc)conf_sel_sel, 0);
	
	twid = XmSelectionBoxGetChild (conf_sel, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild (twid);

	xms = XmStringCreateLtoR ("Conference", XmFONTLIST_DEFAULT_TAG);
	twid = XmSelectionBoxGetChild (conf_sel, XmDIALOG_SELECTION_LABEL);
	XtSetArg (args[0], XmNlabelString, xms);
	XtSetValues (twid, args, 1);
	XmStringFree (xms);
	}

    XmListDeleteAllItems (conf_list);

    for (a = 0; a < max_conf; a++) {
	if (conf_skip_flg == 0) {
	    if (qwk_areas[a].nmsg == 0)
		continue;
	    }
	
	sprintf (tbuf, "%d %s", qwk_areas[a].num, qwk_areas[a].name);
	xms = XmStringCreateLtoR (tbuf, XmFONTLIST_DEFAULT_TAG);
	XmListAddItem (conf_list, xms, 0);
	XmStringFree (xms);
	}

    XtManageChild (conf_sel);
}

/*========================================================================*
 * VIEW_MSG_SEL -- Create/Manage the message selection dialog
 *========================================================================*/

view_msg_sel ()
{
Arg		args[5];
Widget		twid;
XmString	xms;
XmString	item;

    if (msg_sel == NULL) {
	xms = XmStringCreateLtoR ("Message Selection",
				XmFONTLIST_DEFAULT_TAG);

	item = XmStringCreateLtoR ("From                      To"
	"                        Subject                       ",
	XmFONTLIST_DEFAULT_TAG);

	XtSetArg (args[0], XmNdialogTitle, xms);
	XtSetArg (args[1], XmNlistLabelString, item);
	XtSetArg (args[2], XmNresizePolicy, XmRESIZE_NONE);
	msg_sel = XmCreateSelectionDialog (toplevel, "Sel", args, 3);
	XmStringFree (xms);
	XmStringFree (item);

	msg_list = XmSelectionBoxGetChild (msg_sel, XmDIALOG_LIST);
	XtAddCallback (msg_sel, XmNokCallback,
			(XtCallbackProc)msg_press, 0);
	XtAddCallback (msg_list, XmNsingleSelectionCallback,
			(XtCallbackProc)msg_sel_sel, 0);
	XtAddCallback (msg_list, XmNbrowseSelectionCallback,
			(XtCallbackProc)msg_sel_sel, 0);
	
	twid = XmSelectionBoxGetChild (msg_sel, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild (twid);
	twid = XmSelectionBoxGetChild (msg_sel, XmDIALOG_APPLY_BUTTON);
	XtUnmanageChild (twid);

	xms = XmStringCreateLtoR ("Message", XmFONTLIST_DEFAULT_TAG);
	twid = XmSelectionBoxGetChild (msg_sel, XmDIALOG_SELECTION_LABEL);
	XtSetArg (args[0], XmNlabelString, xms);
	XtSetValues (twid, args, 1);
	XmStringFree (xms);
	}

    XmListDeleteAllItems (msg_list);

    build_new_msg_list ();

    XtManageChild (msg_sel);
}

/*========================================================================*
 * BUILD_NEW_MSG_LIST -- Build message list for new conference
 *========================================================================*/

build_new_msg_list ()
{
FILE		*fpi;
int		a;
char		tbuf[256];
QWKHDR		hdr;
XmString	xms;


    sprintf (tbuf, "%s/%d.cnf", bbs_path, qwk_areas[cur_conf].num);

    if ((fpi = fopen (tbuf, "r")) == NULL)
	return;

    a = 0;
    for (;;) {
	if (fread (&hdr, sizeof (hdr), 1, fpi) <= 0) {
	    fclose (fpi);
	    return;
	    }
	while (hdr.numchunks) {
	    fread (tbuf, QWK_BLK, 1, fpi);
	    hdr.numchunks--;
	    }

	sprintf (tbuf, "%-25.25s %-25.25s %-25.25s",
			hdr.from, hdr.to, hdr.subj);

	xms = XmStringCreateLtoR (tbuf, XmFONTLIST_DEFAULT_TAG);
	XmListAddItem (msg_list, xms, 0);
	XmStringFree (xms);
	}
}

/*========================================================================*
 * $Log: unpack.c,v $
 * Revision 1.1  1994/07/24  17:49:35  ross
 * Initial revision
 *
 *========================================================================*/

